# -----------------------------------------------------------------------------
# xtablelist_demo.tcl ---
# -----------------------------------------------------------------------------
# (c) 2016, Johann Oberdorfer - Engineering Support | CAD | Software
#     johann.oberdorfer [at] gmail.com
#     www.johann-oberdorfer.eu
# -----------------------------------------------------------------------------
# This source file is distributed under the BSD license.
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#   See the BSD License for more details.
# -----------------------------------------------------------------------------
# Purpose:
#  A TclOO class template to extend tablelist functionality.
#  Might be usefull as a starting point.
# -----------------------------------------------------------------------------
# TclOO naming conventions:
# public methods  - starts with lower case declaration names, whereas
# private methods - starts with uppercase naming, so we use CamelCase ...
# -----------------------------------------------------------------------------

# for development: try to find autoscroll, etc ...
set dir [file normalize [file dirname [info script]]]

# where to find required packages...
set auto_path [linsert $auto_path 0 [file join $dir "."]]
set auto_path [linsert $auto_path 0 [file join $dir "../../00-lib"]]


package require Tk
package require TclOO

package require autoscroll
package require tablelist_tile

package require xtablelist


# ---------
# demo code
# ---------
catch {console show}


# 1st example:

set t [xtablelist::xtablelist .t \
		-showseparators "yes" \
		-labelcommand "tablelist::sortByColumn" \
		-width 80 -height 10 \
		-xfilesave 1 \
		-xfileread 1 \
		-xshowbuttons 1 \
		-xrefreshpostcmd {puts "Refresh postcommand done."}]

pack $t -fill both -expand true


# maybe some configuration changes later on ...
$t configure \
		-xmoveovereffect "yes"

# create some random test data...

set header {
		{"ID" 10 left}
		{"Category"      22 left}
		{"test-column"   16 left}
		{"Hello\\nWorld" 10 left}
		{"test"          "hidden" center}
		{"last\\ncolumn" 10 left}
	}

$t configure \
		-xtabheader $header

set data_list {}
set cnt 0

while {$cnt < 40} {
	lappend data_list \
			[list $cnt \
			[expr {$cnt +3}] [expr {$cnt +4}] \
			[expr {$cnt +1}] [expr {$cnt +2}] \
			[expr {$cnt +5}]]
	incr cnt
}

foreach item $data_list {
	$t insert end $item
}



# 2nd example:

proc SelectPostCmd {t} {

	set sel [$t selection get]
	puts $sel

}


set t1 [xtablelist::xtablelist .t1 \
		-showseparators "yes" \
		-labelcommand "tablelist::sortByColumn" \
		-width 80 -height 10 \
		-xfilesave 1 \
		-xfileread 0 \
		-xshowbuttons 0 \
		-xrefreshpostcmd {puts "Refresh postcommand done."} \
		-xselectpostcmd "SelectPostCmd .t1"]

pack $t1 -fill both -expand true





set header {
		{"ID" 10 left}
		{"Info_Procs" 20 left}
		{"Body"       hidden left}
	}

$t1 configure \
		-snipstring "..." \
		-xtabheader $header

set data_list {}

set cnt 0
foreach p [lsort -dictionary [info procs]] {
	# set body [string map {"\n" " "} [info body $p]]
	
	set row [list $cnt $p [string trim [info body $p]]]
	$t1 insert end $row
	incr cnt
}


puts "________________"
puts [$t cget -xfilesave]
puts "________________"


puts "________________"
puts [$t1 cget -xfilesave]
puts "________________"

