# -----------------------------------------------------------------------------
# xtablelist_popupmenu.tcl ---
# -----------------------------------------------------------------------------
# (c) 2016, Johann Oberdorfer - Engineering Support | CAD | Software
#     johann.oberdorfer [at] gmail.com
#     www.johann-oberdorfer.eu
# -----------------------------------------------------------------------------
# This source file is distributed under the BSD license.
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#   See the BSD License for more details.
# -----------------------------------------------------------------------------
# Purpose:
#  A TclOO class template to extend tablelist functionality.
# -----------------------------------------------------------------------------


oo::define XTableListClass {
	
	# unused
	method showColumnByIndex {idx} {
		my variable tblwidget
		my variable widgetCompounds
	
		set cnt 0
		foreach column [my GetTblColumnNames] {
			if {$cnt == $idx} {
				$tblwidget columnconfigure $cnt -hide false
				set widgetCompounds(hide_column,$cnt) 0
			}
			incr cnt
		}
	}
	
	
	# sort the list based on a particular column
	method SortCmd {index} {
		my variable tblwidget
		my variable widgetCompounds
	
		$tblwidget sortbycolumn $index
		set widgetCompounds(sortby) $index
	}


	method HideColumnCmd {index} {
		my variable tblwidget
		my variable widgetCompounds

		# hiding all columns doesn't make sense ...
		if {[llength [my GetOnlyVisibleTblColumnNames]] == 1} {
			tk_messageBox \
				-parent $tblwidget \
				-title "Warning" \
				-icon "warning" \
				-message "Cannot hide all columns!" \
				-type ok
			return
		}
		
		if { $widgetCompounds(hide_column,$index) } {
			$tblwidget columnconfigure $index -hide true
		} else {
			$tblwidget columnconfigure $index -hide false
		}
	}

	
	method ShowAllColumnsCmd {} {
		my variable tblwidget
		my variable widgetCompounds
	
		set cnt 0
		foreach column [my GetTblColumnNames] {
			$tblwidget columnconfigure $cnt -hide false
			set widgetCompounds(hide_column,$cnt) 0
			incr cnt
		}
	}

	
	method showContextMenu {colx coly rootx rooty} {
		
		# x,y are the coordinates relative to the upper-left corner of the
		# listbox; rootx,rooty are screen coordinates (for knowing where
		# to place the menu). w is the name of the mclistbox widget that was
		# clicked on.
		
		my variable tblwidget
		my variable widgetOptions
		my variable widgetCompounds
		
		# re-create widget all the time:
		set contextMenu $tblwidget.tblContextMenu
		catch {destroy $contextMenu}
		
		set columnLabel [$tblwidget columncget $colx -title]
		set colIndex $colx
		set column $columnLabel
		
	
		set m [menu $contextMenu -tearoff 1]
		
		if { [my GetBoolean $widgetOptions(-xsortoption)] } {
			
			menu $m.sort_by -tearoff 1
			$m add cascade \
					-label "Sort by ..." \
					-menu $m.sort_by
			
			set cnt 0
			set cbrk_cnt 0
			set col_names [my GetOnlyVisibleTblColumnNames]
			
			foreach i $col_names {
				
				# manage multi column menu, otherwise the menu limits
				# would exceed screen dimensions
				# ---------------------------------------------------
				if {[llength $col_names] > 39} {
					if {$cbrk_cnt < 10} {
						incr cbrk_cnt
						set cbrk 0
					} else {
						set cbrk_cnt 1
						set cbrk 1
					}
				} else {set cbrk 0}
				# ---------------------------------------------------
				
				$m.sort_by add radiobutton \
						-variable "[namespace current]::widgetCompounds(sortby)" \
						-value $cnt \
						-label $i \
						-command "[namespace code {my SortCmd}] $cnt" \
						-columnbreak $cbrk
				incr cnt
			}
			
			$m add command \
					-label "Sort by: $columnLabel" \
					-command "[namespace code {my SortCmd}] $colIndex"
			
			$m add separator
		}
		
		# --------------------------------------------------
		# *only* show context-menu if specified by argument:
		# --------------------------------------------------

		if { [my GetBoolean $widgetOptions(-xhideoption)] } {
		
			menu $m.hide_column -tearoff 1
			$m add cascade \
					-label "Hide column..." \
					-menu $m.hide_column
			
			# $m.hide_column add command \
			#	-label "Hide column:" -state disabled

			set cnt 0
			set cbrk_cnt 0
			set col_names [my GetOnlyVisibleTblColumnNames]
			
			
			foreach i $col_names {
				
				# manage multi column menu, otherwise the menu limits
				# would exceed screen dimensions
				# ---------------------------------------------------
				if {[llength $col_names] > 39} {
					if {$cbrk_cnt < 10} {
						incr cbrk_cnt
						set cbrk 0
					} else {
						set cbrk_cnt 1
						set cbrk 1
					}
				} else {set cbrk 0}
				# ---------------------------------------------------
				
				set colLabel [$tblwidget columncget $cnt -title]

				$m.hide_column add checkbutton \
						-variable "[namespace current]::widgetCompounds(hide_column,$cnt)" \
						-label $colLabel \
						-command "[namespace code {my HideColumnCmd}] $cnt" \
						-columnbreak $cbrk
				incr cnt
			}
			
			$m add command \
					-label "Hide column: $columnLabel" \
					-command "\
						set [namespace current]::widgetCompounds(hide_column,$colIndex) 1;
						[namespace code {my HideColumnCmd}] $colIndex"
			
			# separator doesn't work in column mode
			if {[llength $col_names] <= 39} {
				$m.hide_column add separator
			}

			foreach m_item [list $m $m.hide_column] {
				
				$m_item add command \
						-label "Show All" \
						-command "[namespace code {my ShowAllColumnsCmd}]"
			}
			
			$m add separator
		}
		
		# manage visualisation mode
		# [$w get 0 end] is too slow if the listbox is pretty full...
		
		if {[info exists widgetCompounds(mstate)]} {
			set st $widgetCompounds(mstate)
		} else {
			set st disabled
		}

		$m add command \
			-label "File Operations:" -state disabled

		if { [my GetBoolean $widgetOptions(-xfilesave)] } {
			
			$m add command \
					-label "Save to file..." \
					-command "[namespace code {my Save2File}]" \
					-state $st
		}

		if { [my GetBoolean $widgetOptions(-xfileread)] } {
			
			$m add command \
					-label "Open from file..." \
					-command "[namespace code {my OpenFromFile}]" \
					-state $st
		}
		
		# -----------------------
		tk_popup $m $rootx $rooty
		# -----------------------
		
		# make sure, popup-window stays on top
		# when using "tear-off" the 1st time
		wm withdraw $m
		wm deiconify $m
	}
}