# -----------------------------------------------------------------------------
# xtablelist_readwrite.tcl ---
# -----------------------------------------------------------------------------
# (c) 2016, Johann Oberdorfer - Engineering Support | CAD | Software
#     johann.oberdorfer [at] gmail.com
#     www.johann-oberdorfer.eu
# -----------------------------------------------------------------------------
# This source file is distributed under the BSD license.
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#   See the BSD License for more details.
# -----------------------------------------------------------------------------
# Purpose:
#  A TclOO class template to extend tablelist functionality.
# -----------------------------------------------------------------------------

oo::define XTableListClass {

	method GetAllVisibleItems {} {
	
		# only take visible columns into account
		# "...what you see is what you get..."

		my variable tblwidget

		set is_hidden {}
		set cnt 0
		
		foreach column [my GetTblColumnNames] {
			lappend is_hidden [$tblwidget columncget $cnt -hide]
			incr cnt
		}
		
		set lst {}
		foreach i [$tblwidget get 0 end] {
			set cnt 0
			set sublst {}
			
			foreach xitem $i {
				if {[lindex $is_hidden $cnt] != 1} {
					lappend sublst $xitem
				}
				incr cnt
			}
			
			lappend lst $sublst
		}
		
		return $lst
	}

	method Write2File {fname} {
		my variable tblwidget
		my variable widgetCompounds
		
		if {$fname == ""} {
			return
		}
		
		set lst [my GetAllVisibleItems]
		set fp  [open $fname "w"]
		
		foreach i $lst {
			# regsub -all "\{" $i "" i
			# regsub -all "\}" $i "" i
			
			set str ""
			foreach item $i {
				if {$str == ""} {
					set str $item
				} else {
					append str $widgetCompounds(DELIM)
					append str $item
				}
			}
			
			puts $fp $str
		}
		close $fp
	}
	
	method Save2File {} {
		my variable tblwidget
		
		#-now we use this one-
		set fname [xtablelist::getfilename \
				-parent      $tblwidget \
				-title       "Select file where to store Listbox information:" \
				-oktext      " Save to File... "]
		
		if {[string length $fname] > 0} {
			my Write2File $fname
		}
	}
	
	method OpenFromFile {} {
		my variable tblwidget
		my variable widgetCompounds
		
		set in_file [ \
				tk_getOpenFile \
				-parent $tblwidget \
				-title "Select file:" \
				-initialdir $widgetCompounds(default_dir) \
				-defaultextension {.txt} \
				-filetypes  {{"Text-Files" "*.txt*"} {all "*.*"}} \
				-initialfile ""]
		
		# read values from file and uptdate listbox ...
		if {$in_file != ""} {
			
			set data {}
			set fp [open $in_file "r"]
			
			while {![eof $fp]} {
				set i [string trim [gets $fp]]
				
				if {$i != ""} {
					lappend data [split $i $widgetCompounds(DELIM)]
				}
			}
			close $fp
			
			# foreach i $data {puts "--> $i <--"}
			my addValues $data
			
			# finally execute post command
			catch {
				$widgetOptions(-xfilereadpostcmd)
			}
		}
	}
	
	method Save2Excel {} {
		my variable tblwidget
		my variable widgetCompounds

		set excelfile \
		[xtablelist::getfilename \
			-parent $tblwidget \
			-title "Select Excel file (for mail attachement):" \
			-filetypes {{"Excel-File" "*.xls*"} {all "*.*"}} \
			-defaultext ".xls" \
			-oktext "Save Excel-File... "]

		if {$excelfile == ""} {
			return
		}

		# read (visible!) header
		# ----------------------
		set visible_header {}
		set cnt 0
		foreach column [my GetTblColumnNames] {
			if { ![$tblwidget columncget $cnt -hide] } {
				lappend visible_header [$tblwidget columncget $cnt -title]
			}
			incr cnt
		}

		# read current (visible!) tablelist items
		# ---------------------------------------
		set data [my GetAllVisibleItems]

		set lst [list $visible_header]
		foreach i $data {
			lappend lst $i
		}

		# file operations
		# ---------------
		set of  [open $widgetCompounds(tmp_file) "w"]

		foreach sublst $lst {
			set str ""
			set cnt 0

			foreach i $sublst {
				if {$cnt == 0} {
					set str "$i"
				} else {
					set str "$str$widgetCompounds(DELIM)$i"
				}

				incr cnt
			}
    
			if {[string length $str] != 0} {
				puts $of $str
			}
		}
		close $of

		# system independend excel sheet generation:
		Cvs2Biff::convert $widgetCompounds(tmp_file) $excelfile
		file delete -force $widgetCompounds(tmp_file)
	}
}